@@ -13,6 +13,10 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import netaddr
+import os
+import random
+
 from oslo_log import log as logging
 from tempest.common import compute
 from tempest.common import utils
@@ -22,28 +26,57 @@ from tempest.lib.common.utils import data_utils
 from tempest.lib.common.utils import test_utils
 from tempest.lib import decorators
 
+from neutron_lib.utils import test
+
 from networking_bgpvpn_tempest.tests import base
 from networking_bgpvpn_tempest.tests.scenario import manager
 
+from oslo_concurrency import lockutils
+
+
 CONF = config.CONF
 LOG = logging.getLogger(__name__)
-RT1 = '64512:1'
-RT2 = '64512:2'
 NET_A = 'A'
 NET_A_BIS = 'A-Bis'
 NET_B = 'B'
-NET_A_S1 = '10.101.11.0/24'
-NET_A_S2 = '10.101.12.0/24'
-NET_B_S1 = '10.102.21.0/24'
-NET_B_S2 = '10.102.22.0/24'
-IP_A_S1_1 = '10.101.11.10'
-IP_B_S1_1 = '10.102.21.20'
-IP_A_S1_2 = '10.101.11.30'
-IP_B_S1_2 = '10.102.21.40'
-IP_A_S1_3 = '10.101.11.50'
-IP_B_S1_3 = '10.102.21.60'
-IP_A_S2_1 = '10.101.12.50'
-IP_B_S2_1 = '10.102.22.60'
+NET_C = 'C'
+
+if "SUBNETPOOL_PREFIX_V4" in os.environ:
+    subnet_base = netaddr.IPNetwork(os.environ['SUBNETPOOL_PREFIX_V4'])
+    if subnet_base.prefixlen > 21:
+        raise Exception("if SUBNETPOOL_PREFIX_V4 is set, it needs to offer "
+                        "space for at least 8 /24 subnets")
+else:
+    subnet_base = netaddr.IPNetwork("10.100.0.0/16")
+
+
+def assign_24(idx):
+    # how many addresses in a /24:
+    range_size = 2 ** (32 - 24)
+    return netaddr.cidr_merge(
+        subnet_base[range_size * idx:range_size * (idx + 1)]
+        )[0]
+
+
+S1A = assign_24(1)
+S2A = assign_24(2)
+S1B = assign_24(4)
+S2B = assign_24(6)
+S1C = assign_24(6)
+NET_A_S1 = str(S1A)
+NET_A_S2 = str(S2A)
+NET_B_S1 = str(S1B)
+NET_B_S2 = str(S2B)
+NET_C_S1 = str(S1C)
+IP_A_S1_1 = str(S1A[10])
+IP_B_S1_1 = str(S1B[20])
+IP_C_S1_1 = str(S1C[30])
+IP_A_S1_2 = str(S1A[30])
+IP_B_S1_2 = str(S1B[40])
+IP_A_S1_3 = str(S1A[50])
+IP_B_S1_3 = str(S1B[60])
+IP_A_S2_1 = str(S2A[50])
+IP_B_S2_1 = str(S2B[60])
 IP_A_BIS_S1_1 = IP_A_S1_1
 IP_A_BIS_S1_2 = IP_A_S1_2
 IP_A_BIS_S1_3 = IP_A_S1_3
@@ -51,6 +84,18 @@ IP_A_BIS_S2_1 = IP_A_S2_1
 
 
 class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
+
+    @classmethod
+    def setUpClass(cls):
+        super(TestBGPVPNBasic, cls).setUpClass()
+        cls._rt_index = 0
+
+    @classmethod
+    @lockutils.synchronized('bgpvpn')
+    def new_rt(cls):
+        cls._rt_index += 1
+        return "64512:%d" % cls._rt_index
+
     def setUp(self):
         super(TestBGPVPNBasic, self).setUp()
         self.servers_keypairs = {}
@@ -61,6 +106,10 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
         self.subnets = {}
         self.server_fips = {}
         self._create_security_group_for_test()
+        self.RT1 = self.new_rt()
+        self.RT2 = self.new_rt()
+        self.RT3 = self.new_rt()
+        self.RT4 = self.new_rt()
 
     @decorators.idempotent_id('afdd6cad-871a-4343-b97b-6319c76c815d')
     @utils.services('compute', 'network')
@@ -234,12 +283,14 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
         """
         self._create_networks_and_subnets()
         self._create_servers()
-        self._create_l3_bgpvpn(rts=[RT1], import_rts=[], export_rts=[])
+        self._create_l3_bgpvpn(rts=[self.RT1], import_rts=[],
+                               export_rts=[])
         self._associate_all_nets_to_bgpvpn()
         self._associate_fip_and_check_l3_bgpvpn()
-        self._update_l3_bgpvpn(rts=[], import_rts=[RT1], export_rts=[RT2])
+        self._update_l3_bgpvpn(rts=[], import_rts=[self.RT1],
+                               export_rts=[self.RT2])
         self._check_l3_bgpvpn(should_succeed=False)
-        self._update_l3_bgpvpn(rts=[RT1], import_rts=[], export_rts=[])
+        self._update_l3_bgpvpn(rts=[self.RT1], import_rts=[], export_rts=[])
         self._check_l3_bgpvpn()
 
     @decorators.idempotent_id('bf417cad-0bc4-446a-b367-850aa619ca4f')
@@ -262,12 +313,13 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
         """
         self._create_networks_and_subnets()
         self._create_servers()
-        self._create_l3_bgpvpn(rts=[RT1], import_rts=[], export_rts=[])
+        self._create_l3_bgpvpn(rts=[self.RT1], import_rts=[], export_rts=[])
         self._associate_all_nets_to_bgpvpn()
         self._associate_fip_and_check_l3_bgpvpn()
-        self._update_l3_bgpvpn(rts=[RT1], import_rts=[RT1], export_rts=[RT2])
+        self._update_l3_bgpvpn(rts=[self.RT1], import_rts=[self.RT1],
+                               export_rts=[self.RT2])
         self._check_l3_bgpvpn()
-        self._update_l3_bgpvpn(rts=[RT1], import_rts=[], export_rts=[])
+        self._update_l3_bgpvpn(rts=[self.RT1], import_rts=[], export_rts=[])
         self._check_l3_bgpvpn()
 
     @decorators.idempotent_id('08d4f40e-3cec-485b-9da2-76e67fbd9881')
@@ -293,14 +345,15 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
         self._create_servers()
         self.router = self._create_router_and_associate_fip(
             0, self.subnets[NET_A][0])
-        self._create_l3_bgpvpn(rts=[], export_rts=[RT1], import_rts=[RT2])
+        self._create_l3_bgpvpn(rts=[], export_rts=[self.RT1],
+                               import_rts=[self.RT2])
         self.bgpvpn_client.create_network_association(
             self.bgpvpn['id'], self.networks[NET_A]['id'])
         self._check_l3_bgpvpn(should_succeed=False)
         self.bgpvpn_client.create_network_association(
             self.bgpvpn['id'], self.networks[NET_B]['id'])
         self._check_l3_bgpvpn(should_succeed=False)
-        self._update_l3_bgpvpn(rts=[RT1], import_rts=[], export_rts=[])
+        self._update_l3_bgpvpn(rts=[self.RT1], import_rts=[], export_rts=[])
         self._check_l3_bgpvpn()
 
     @decorators.idempotent_id('c8bfd695-f731-47a6-86e3-3dfa492e08e0')
@@ -331,7 +384,8 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
         20. Check that server 2 still can ping server 4
         """
         self._create_networks_and_subnets()
-        self._create_l3_bgpvpn(rts=[], import_rts=[RT1], export_rts=[RT2])
+        self._create_l3_bgpvpn(rts=[], import_rts=[self.RT1],
+                               export_rts=[self.RT2])
         self._create_servers([[self.networks[NET_A], IP_A_S1_1],
                              [self.networks[NET_B], IP_B_S1_1],
                              [self.networks[NET_A], IP_A_S1_2],
@@ -348,7 +402,7 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
             1, self.subnets[NET_B][0])
         self._check_l3_bgpvpn(should_succeed=False)
         self._check_l3_bgpvpn(self.servers[1], self.servers[3])
-        self._update_l3_bgpvpn(rts=[RT1], import_rts=[], export_rts=[])
+        self._update_l3_bgpvpn(rts=[self.RT1], import_rts=[], export_rts=[])
         self._check_l3_bgpvpn()
         self._check_l3_bgpvpn(self.servers[0], self.servers[2])
         self._check_l3_bgpvpn(self.servers[1], self.servers[3])
@@ -381,7 +435,8 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
         20. Check that server 4 still can ping server 2
         """
         self._create_networks_and_subnets()
-        self._create_l3_bgpvpn(rts=[], import_rts=[RT1], export_rts=[RT2])
+        self._create_l3_bgpvpn(rts=[], import_rts=[self.RT1],
+                               export_rts=[self.RT2])
         self._create_servers([[self.networks[NET_A], IP_A_S1_1],
                              [self.networks[NET_B], IP_B_S1_1],
                              [self.networks[NET_A], IP_A_S1_2],
@@ -398,7 +453,7 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
                                                      router_b['id'])
         self._check_l3_bgpvpn(should_succeed=False)
         self._check_l3_bgpvpn(self.servers[3], self.servers[1])
-        self._update_l3_bgpvpn(rts=[RT1], import_rts=[], export_rts=[])
+        self._update_l3_bgpvpn(rts=[self.RT1], import_rts=[], export_rts=[])
         self._check_l3_bgpvpn()
         self._check_l3_bgpvpn(self.servers[0], self.servers[2])
         self._check_l3_bgpvpn(self.servers[3], self.servers[1])
@@ -449,17 +504,15 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
         self._setup_http_server(2)
         self._setup_ip_forwarding(1)
         self._setup_ip_forwarding(2)
-        alternative_loopback_ip = '192.168.0.1'
-        alternative_loopback_cidr = '192.168.0.0/24'
-        self._setup_ip_address(1, alternative_loopback_cidr)
-        self._setup_ip_address(2, alternative_loopback_cidr)
+        self._setup_ip_address(1, IP_C_S1_1)
+        self._setup_ip_address(2, IP_C_S1_1)
 
         primary_port_routes = [{'type': 'prefix',
                                 'local_pref': 200,
-                                'prefix': alternative_loopback_cidr}]
+                                'prefix': NET_C_S1}]
         alternate_port_routes = [{'type': 'prefix',
                                   'local_pref': 100,
-                                  'prefix': alternative_loopback_cidr}]
+                                  'prefix': NET_C_S1}]
 
         self.bgpvpn_client.create_network_association(
             self.bgpvpn['id'], self.networks[NET_A]['id'])
@@ -480,7 +533,7 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
                                        self.servers[2]['id'])
 
         self._check_l3_bgpvpn_by_specific_ip(
-            to_server_ip=alternative_loopback_ip,
+            to_server_ip=IP_C_S1_1,
             validate_server=destination_srv_1)
 
         self.bgpvpn_client.update_port_association(
@@ -491,7 +544,7 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
             routes=primary_port_routes)
 
         self._check_l3_bgpvpn_by_specific_ip(
-            to_server_ip=alternative_loopback_ip,
+            to_server_ip=IP_C_S1_1,
             validate_server=destination_srv_2)
 
     @decorators.idempotent_id('f762e6ac-920e-4d0f-aa67-02bdd4ab8433')
@@ -501,8 +554,8 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
 
         1. Create networks A with subnet S1 and S2
         2. Create networks A-Bis with subnet S1 and S2 (like for network A)
-        3. Create L3 BGPVPN for network A with RT1
-        4. Create L3 BGPVPN for network A-Bis with RT2
+        3. Create L3 BGPVPN for network A with self.RT1
+        4. Create L3 BGPVPN for network A-Bis with self.RT2
         5. Associate network A to a given L3 BGPVPN
         6. Associate network A-Bis to a given L3 BGPVPN
         7. Start up server 1 in network A and subnet S1
@@ -525,9 +578,9 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
                                           [[NET_A_S1, NET_A_S2],
                                            [NET_A_S1, NET_A_S2]])
         bgpvpn_a = self._create_l3_bgpvpn(name='test-l3-bgpvpn-a',
-                                          rts=[RT1])
+                                          rts=[self.RT1])
         bgpvpn_a_bis = self._create_l3_bgpvpn(name='test-l3-bgpvpn-a-bis',
-                                              rts=[RT2])
+                                              rts=[self.RT2])
         self.bgpvpn_client.create_network_association(
             bgpvpn_a['id'], self.networks[NET_A]['id'])
         self.bgpvpn_client.create_network_association(
@@ -582,9 +635,6 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
         13. Update created before port association by routes removal
         14. Check that server 1 cannot ping server's 2 alternative ip
         """
-        alternative_loopback_ip = '192.168.0.1'
-        alternative_loopback_cidr = '192.168.0.0/24'
-
         self._create_networks_and_subnets(port_security=False)
         self._create_l3_bgpvpn()
         self._create_servers([[self.networks[NET_A], IP_A_S1_1],
@@ -598,25 +648,101 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
         # preliminary check that no connectivity to 192.168.0.1 initially
         # exists
         self._check_l3_bgpvpn_by_specific_ip(
-            should_succeed=False, to_server_ip=alternative_loopback_ip)
+            should_succeed=False, to_server_ip=IP_C_S1_1)
 
         self._setup_ip_forwarding(1)
-        self._setup_ip_address(1, alternative_loopback_cidr)
+        self._setup_ip_address(1, IP_C_S1_1)
         self.bgpvpn_client.create_network_association(
             self.bgpvpn['id'], self.networks[NET_A]['id'])
         port_id = self.ports[self.servers[1]['id']]['id']
         port_routes = [{'type': 'prefix',
-                        'prefix': alternative_loopback_cidr}]
+                        'prefix': NET_C_S1}]
         body = self.bgpvpn_client.create_port_association(self.bgpvpn['id'],
                                                           port_id=port_id,
                                                           routes=port_routes)
         port_association = body['port_association']
         self._check_l3_bgpvpn_by_specific_ip(
-            to_server_ip=alternative_loopback_ip)
+            to_server_ip=IP_C_S1_1)
         self.bgpvpn_client.update_port_association(
             self.bgpvpn['id'], port_association['id'], routes=[])
         self._check_l3_bgpvpn_by_specific_ip(
-            should_succeed=False, to_server_ip=alternative_loopback_ip)
+            should_succeed=False, to_server_ip=IP_C_S1_1)
+
+    @decorators.idempotent_id('d92a8a18-c4d0-40d5-8592-713d7dae7d25')
+    @utils.services('compute', 'network')
+    @utils.requires_ext(extension='bgpvpn-routes-control', service='network')
+    @test.unstable_test("bug 1780205")
+    def test_port_association_many_bgpvpn_routes(self):
+        """This test checks port association in BGPVPN.
+
+        1. Create networks A and B with their respective subnets
+        2. Create L3 BGPVPN
+        3. Create router and connect it to network A
+        4. Create router and connect it to network B
+        5. Start up server 1 in network A
+        6. Start up server 2 in network B
+        7. Give a FIP to all servers
+        8. Configure ip forwarding on server 2
+        9. Configure alternative loopback address on server 2
+        10. Associate network A to a given L3 BGPVPN
+        11. Associate port of server 2 to a given L3 BGPVPN
+        12. Check that server 1 can ping server's 2 alternative ip
+        13. Update created before port association by routes removal
+        14. Check that server 1 cannot ping server's 2 alternative ip
+        """
+        AMOUNT_LOOPBACKS = 90
+        START_IP_LOOPBACKS = 31
+        SAMPLE_SIZE = 10
+        LOOPBACKS = [str(ip) for ip in
+                     S1C[START_IP_LOOPBACKS:
+                     START_IP_LOOPBACKS + AMOUNT_LOOPBACKS]]
+        SUB_LOOPBACKS = [LOOPBACKS[0], LOOPBACKS[-1]]
+
+        self._create_networks_and_subnets(port_security=False)
+        self._create_l3_bgpvpn()
+        self._create_servers([[self.networks[NET_A], IP_A_S1_1],
+                              [self.networks[NET_B], IP_B_S1_1]],
+                             port_security=False)
+        self._create_fip_router(subnet_id=self.subnets[NET_A][0]['id'])
+        self._create_fip_router(subnet_id=self.subnets[NET_B][0]['id'])
+        self._associate_fip(0)
+        self._associate_fip(1)
+
+        for ip in SUB_LOOPBACKS:
+            LOG.debug("Preliminary check that no connectivity exist")
+            self._check_l3_bgpvpn_by_specific_ip(
+                should_succeed=False, to_server_ip=ip)
+
+        self._setup_ip_forwarding(1)
+
+        self._setup_range_ip_address(1, LOOPBACKS)
+
+        self.bgpvpn_client.create_network_association(
+            self.bgpvpn['id'], self.networks[NET_A]['id'])
+        port_id = self.ports[self.servers[1]['id']]['id']
+        port_routes = [{'type': 'prefix',
+                        'prefix': ip + "/32"}
+                       for ip in LOOPBACKS]
+
+        body = self.bgpvpn_client.create_port_association(self.bgpvpn['id'],
+                                                          port_id=port_id,
+                                                          routes=port_routes)
+        port_association = body['port_association']
+
+        for ip in random.sample(LOOPBACKS, SAMPLE_SIZE):
+            LOG.debug("Check that server 1 can "
+                      "ping server 2 alternative ip %s" % ip)
+            self._check_l3_bgpvpn_by_specific_ip(
+                to_server_ip=ip)
+
+        self.bgpvpn_client.update_port_association(
+            self.bgpvpn['id'], port_association['id'], routes=[])
+
+        for ip in SUB_LOOPBACKS:
+            LOG.debug("Check that server 1 can't "
+                      "ping server 2 alternative ip %s" % ip)
+            self._check_l3_bgpvpn_by_specific_ip(
+                should_succeed=False, to_server_ip=ip)
 
     @decorators.idempotent_id('9c3280b5-0b32-4562-800c-0b50d9d52bfd')
     @utils.services('compute', 'network')
@@ -639,9 +765,6 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
         13. Remove created before port association
         14. Check that server 1 cannot ping server's 2 alternative ip
         """
-        alternative_loopback_ip = '192.168.0.1'
-        alternative_loopback_cidr = '192.168.0.0/24'
-
         self._create_networks_and_subnets(port_security=False)
         self._create_l3_bgpvpn()
         self._create_servers([[self.networks[NET_A], IP_A_S1_1],
@@ -655,25 +778,149 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
         # preliminary check that no connectivity to 192.168.0.1 initially
         # exists
         self._check_l3_bgpvpn_by_specific_ip(
-            should_succeed=False, to_server_ip=alternative_loopback_ip)
+            should_succeed=False, to_server_ip=IP_C_S1_1)
 
         self._setup_ip_forwarding(1)
-        self._setup_ip_address(1, alternative_loopback_cidr)
+        self._setup_ip_address(1, IP_C_S1_1)
         self.bgpvpn_client.create_network_association(
             self.bgpvpn['id'], self.networks[NET_A]['id'])
         port_id = self.ports[self.servers[1]['id']]['id']
         port_routes = [{'type': 'prefix',
-                        'prefix': alternative_loopback_cidr}]
+                        'prefix': NET_C_S1}]
         body = self.bgpvpn_client.create_port_association(self.bgpvpn['id'],
                                                           port_id=port_id,
                                                           routes=port_routes)
         port_association = body['port_association']
         self._check_l3_bgpvpn_by_specific_ip(
-            to_server_ip=alternative_loopback_ip)
+            to_server_ip=IP_C_S1_1)
         self.bgpvpn_client.delete_port_association(
             self.bgpvpn['id'], port_association['id'])
         self._check_l3_bgpvpn_by_specific_ip(
-            should_succeed=False, to_server_ip=alternative_loopback_ip)
+            should_succeed=False, to_server_ip=IP_C_S1_1)
+
+    @decorators.idempotent_id('73f629fa-fdae-40fc-8a7e-da3aedcf797a')
+    @utils.services('compute', 'network')
+    @utils.requires_ext(extension='bgpvpn-routes-control', service='network')
+    def test_bgpvpn_port_association_bgpvpn_route(self):
+        """Test port association routes of type 'bgpvpn'
+
+        In this test we use a Port Association with a 'bgpvpn' route
+        to have VM 1 in network A, reach a VM 3 in network C via the Port
+        of a VM 2 (on network B), and vice-versa. For A->C traffic, one Port
+        Association associates the port of VM 2 to a BGPVPN, with a route of
+        type 'bgpvpn' redistributing routes from network  C. For C->A traffic,
+        another Port Association associates the port of VM 2 to a BGPVPN, with
+        a route of type 'bgpvpn' redistributing routes from network A.
+
+        The use of RT in this test is a bit complex, and would be much
+        simpler if we were using a VM with two interfaces.
+
+        We confirm that VM 1 can join VM 3, and we confirm that the traffic
+        actually goes through VM 2, by turning ip_forwaring off then on in VM2.
+        """
+        # create networks A, B and C with their respective subnets
+        self._create_networks_and_subnets(port_security=False)
+
+        RT_to_A = self.RT1
+        RT_from_A = self.RT2
+        RT_to_C = self.RT3
+        RT_from_C = self.RT4
+
+        # Create L3 BGPVPN for network A
+        bgpvpn_a = self._create_l3_bgpvpn(name="test-vpn-a",
+                                          import_rts=[RT_to_A],
+                                          export_rts=[RT_from_A])
+
+        # Create L3 BGPVPN for network C
+        bgpvpn_c = self._create_l3_bgpvpn(name="test-vpn-c",
+                                          import_rts=[RT_to_C],
+                                          export_rts=[RT_from_C])
+
+        # Create L3 BGPVPN for network B
+        bgpvpn_b = self._create_l3_bgpvpn(name="test-vpn-b",
+                                          import_rts=[RT_from_C, RT_from_A])
+
+        # BGPVPNs to only export into A and C
+        bgpvpn_to_a = self._create_l3_bgpvpn(name="test-vpn-to-a",
+                                             export_rts=[RT_to_A])
+        bgpvpn_to_c = self._create_l3_bgpvpn(name="test-vpn-to-c",
+                                             export_rts=[RT_to_C])
+
+        # create the three VMs
+        self._create_servers([[self.networks[NET_A], IP_A_S1_1],
+                              [self.networks[NET_B], IP_B_S1_1],
+                              [self.networks[NET_C], IP_C_S1_1]],
+                             port_security=False)
+        vm1, vm2, vm3 = self.servers
+
+        # Create one router for each of network A, B, C and give floating
+        # IPs to servers 1, 2, 3
+        self._create_fip_router(subnet_id=self.subnets[NET_A][0]['id'])
+        self._create_fip_router(subnet_id=self.subnets[NET_B][0]['id'])
+        self._create_fip_router(subnet_id=self.subnets[NET_C][0]['id'])
+        self._associate_fip(0)
+        self._associate_fip(1)
+        self._associate_fip(2)
+
+        # disable IP forwarding on VM2
+        self._setup_ip_forwarding(0)
+
+        # connect network A to its BGPVPN
+        self.bgpvpn_client.create_network_association(
+            bgpvpn_a['id'], self.networks[NET_A]['id'])
+
+        # connect network B to its BGPVPN
+        self.bgpvpn_client.create_network_association(
+            bgpvpn_b['id'], self.networks[NET_B]['id'])
+
+        # connect network C to its BGPVPN
+        self.bgpvpn_client.create_network_association(
+            bgpvpn_c['id'], self.networks[NET_C]['id'])
+
+        # create port associations for A->C traffic
+        # (leak routes imported by BGPVPN B -- which happen to include the
+        # routes net C -- into net A)
+        self.bgpvpn_client.create_port_association(
+            bgpvpn_to_a['id'],
+            port_id=self.ports[vm2['id']]['id'],
+            routes=[{'type': 'bgpvpn',
+                     'bgpvpn_id': bgpvpn_b['id']},
+                    ])
+
+        # create port associations for C->A traffic
+        # (leak routes imported by BGPVPN B -- which happen to include the
+        # routes from net A -- into net C)
+        body = self.bgpvpn_client.create_port_association(
+            bgpvpn_to_c['id'],
+            port_id=self.ports[vm2['id']]['id'],
+            routes=[{'type': 'bgpvpn',
+                     'bgpvpn_id': bgpvpn_a['id']}])
+
+        port_association = body['port_association']
+
+        # check that we don't have connectivity
+        # (because destination is supposed to be reachable via VM2, which
+        # still has ip_forwarding disabled)
+        self._check_l3_bgpvpn_by_specific_ip(from_server=vm1,
+                                             to_server_ip=IP_C_S1_1,
+                                             should_succeed=False)
+
+        # enable IP forwarding on VM2
+        self._setup_ip_forwarding(1)
+
+        # VM1 should now be able to join VM2
+        self._check_l3_bgpvpn_by_specific_ip(from_server=vm1,
+                                             to_server_ip=IP_C_S1_1,
+                                             should_succeed=True)
+
+        # remove port association 1
+        self.bgpvpn_client.delete_port_association(self.bgpvpn['id'],
+                                                   port_association['id'])
+
+        # check that connectivity is actually interrupted
+        self._check_l3_bgpvpn_by_specific_ip(from_server=vm1,
+                                             to_server_ip=IP_C_S1_1,
+                                             should_succeed=False)
 
     @decorators.idempotent_id('8478074e-22df-4234-b02b-61257b475b18')
     @utils.services('compute', 'network')
@@ -712,7 +959,8 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
         8. Check that server 1 cannot ping server 2
         """
         self._create_networks_and_subnets()
-        self._create_l3_bgpvpn(rts=[], import_rts=[RT1], export_rts=[RT2])
+        self._create_l3_bgpvpn(rts=[], import_rts=[self.RT1],
+                               export_rts=[self.RT2])
         self._associate_all_nets_to_bgpvpn()
         self._create_servers()
         self._associate_fip_and_check_l3_bgpvpn(should_succeed=False)
@@ -821,11 +1069,12 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
         10. Check that server 1 cannot ping server 2
         """
         self._create_networks_and_subnets()
-        self._create_l3_bgpvpn(rts=[RT1], import_rts=[], export_rts=[])
+        self._create_l3_bgpvpn(rts=[self.RT1], import_rts=[], export_rts=[])
         self._create_servers()
         self._associate_all_nets_to_bgpvpn()
         self._associate_fip_and_check_l3_bgpvpn()
-        self._update_l3_bgpvpn(rts=[], import_rts=[RT1], export_rts=[RT2])
+        self._update_l3_bgpvpn(rts=[], import_rts=[self.RT1],
+                               export_rts=[self.RT2])
         self._check_l3_bgpvpn(should_succeed=False)
 
     @decorators.idempotent_id('fb37a546-7263-4ffe-a42c-77eca377ff1a')
@@ -846,7 +1095,7 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
         10. Check that server 1 cannot ping server 2
         """
         self._create_networks_and_subnets()
-        self._create_l3_bgpvpn(rts=[RT1], import_rts=[], export_rts=[])
+        self._create_l3_bgpvpn(rts=[self.RT1], import_rts=[], export_rts=[])
         self._create_servers()
         self._associate_all_nets_to_bgpvpn()
         self._associate_fip_and_check_l3_bgpvpn()
@@ -860,9 +1109,9 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
     def _create_networks_and_subnets(self, names=None, subnet_cidrs=None,
                                      port_security=True):
         if not names:
-            names = [NET_A, NET_B]
+            names = [NET_A, NET_B, NET_C]
         if not subnet_cidrs:
-            subnet_cidrs = [[NET_A_S1], [NET_B_S1]]
+            subnet_cidrs = [[NET_A_S1], [NET_B_S1], [NET_C_S1]]
         for (name, subnet_cidrs) in zip(names, subnet_cidrs):
             network = self._create_network(
                 namestart=name, port_security_enabled=port_security)
@@ -970,8 +1219,8 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
 
     def _create_l3_bgpvpn(self, name='test-l3-bgpvpn', rts=None,
                           import_rts=None, export_rts=None):
-        if rts is None:
-            rts = [RT1]
+        if rts is None and import_rts is None and export_rts is None:
+            rts = [self.RT1]
         import_rts = import_rts or []
         export_rts = export_rts or []
         self.bgpvpn = self.create_bgpvpn(
@@ -984,7 +1233,7 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
                           bgpvpn=None):
         bgpvpn = bgpvpn or self.bgpvpn
         if rts is None:
-            rts = [RT1]
+            rts = [self.RT1]
         import_rts = import_rts or []
         export_rts = export_rts or []
         LOG.debug('Updating targets in BGPVPN %s', bgpvpn['id'])
@@ -1021,17 +1270,19 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
         ssh_client.exec_command("sudo sysctl -w net.ipv4.ip_forward=1")
 
     def _setup_ip_address(self, server_index, cidr, device=None):
+        self._setup_range_ip_address(server_index, [cidr], device=None)
+
+    def _setup_range_ip_address(self, server_index, cidrs, device=None):
+        MAX_CIDRS = 50
         if device is None:
             device = 'lo'
         server = self.servers[server_index]
         ssh_client = self._setup_ssh_client(server)
-        ssh_client.exec_command(("sudo ip addr add {cidr} "
-                                "dev {dev}").format(cidr=cidr, dev=device))
-
-    def _log_gw_arp_info(self, ssh_client, gateway_ip):
-        output = ssh_client.exec_command('arp -n %s' % gateway_ip)
-        LOG.warning("Ping origin server GW ARP info:")
-        LOG.warning(output)
+        for i in range(0, len(cidrs), MAX_CIDRS):
+            ips = ' '.join(cidrs[i:i + MAX_CIDRS])
+            ssh_client.exec_command(
+                ("for ip in {ips}; do sudo ip addr add $ip "
+                 "dev {dev}; done").format(ips=ips, dev=device))
 
     def _check_l3_bgpvpn(self, from_server=None, to_server=None,
                          should_succeed=True, validate_server=False):
@@ -1069,23 +1320,8 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
             result = self._check_remote_connectivity(ssh_client,
                                                      to_server_ip,
                                                      check_reachable)
-            if check_reachable and not result:
-                allocation = self.ports[from_server['id']]['fixed_ips'][0]
-                subnet_id = allocation['subnet_id']
-                gateway_ip = ''
-                for net_name in self.subnets:
-                    for subnet in self.subnets[net_name]:
-                        if subnet_id == subnet['id']:
-                            gateway_ip = subnet['gateway_ip']
-                            break
-                self._log_gw_arp_info(ssh_client, gateway_ip)
-                ssh_client.exec_command('sudo arp -d %s' % gateway_ip)
-                LOG.warning("Ping origin server GW ARP cleared")
-                ssh_client.ping_host(gateway_ip)
-                self._log_gw_arp_info(ssh_client, gateway_ip)
-                result = self._check_remote_connectivity(ssh_client,
-                                                         to_server_ip,
-                                                         check_reachable)
+            # if a negative connectivity check was unsuccessful (unexpected
+            # ping reply) then try to know more:
             if not check_reachable and not result:
                 try:
                     content = ssh_client.exec_command(
@@ -1097,7 +1333,7 @@ class TestBGPVPNBasic(base.BaseBgpvpnTest, manager.NetworkScenarioTest):
             self.assertTrue(result, msg)
 
             if validate_server and result:
-                # repeating multiple times gives increased ods of avoiding
+                # repeating multiple times gives increased odds of avoiding
                 # false positives in the case where the dataplane does
                 # equal-cost multipath
                 for i in range(0, repeat_validate_server):
